<?php

namespace Drupal\context_redirect\EventSubscriber;

use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Redirect the user to a path / url based on a redirect reaction.
 */
class ContextRedirectResponseSubscriber implements EventSubscriberInterface {

  /**
   * If the context can be evaluated at the request
   * (ex. a path and role condition), then redirect right away.
   *
   * @param RequestEvent $event
   *   The request event.
   */
  public function redirectRequest(RequestEvent $event) {
    if ($redirect = $this->executeRedirect()) {
      $event->setResponse($redirect);
    }
  }

  /**
   * If there are other conditions from the response,
   * override the prior response with our redirect response.
   *
   * @param ResponseEvent $event
   *   The response event.
   */
  public function redirectResponse(ResponseEvent $event) {
    $response = $event->getResponse();
    // Don't try to redirect again.
    if (!($response instanceof RedirectResponse)) {
      if ($redirect = $this->executeRedirect()) {
        // Override the response with a redirect.
        $event->setResponse($redirect);
      }
    }
  }

  /**
   * Return the first redirect from the context reaction if any are available.
   *
   * @return RedirectResponse|bool
   */
  protected function executeRedirect() {
    if ($contextManager = \Drupal::service('context.manager')) {
      foreach ($contextManager->getActiveReactions('redirect') as $reaction) {
        $response = $reaction->execute();
        if ($response instanceof RedirectResponse) {
          // Return the first RedirectResponse from a context.
          return $response;
        }
      }
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::REQUEST][] = ['redirectRequest'];
    $events[KernelEvents::RESPONSE][] = ['redirectResponse'];
    return $events;
  }

}
